#include "item.h"

#include "level.h"
#include "main.h"
#include "stone.h"

Item::Item(unsigned int id) {
  this->remote = id!=0;
  this->id = id;
  this->id = level->register_item(this);
}

Item::~Item() {
  level->unregister(this);
}

unsigned int Item::get_id() {
  return id;
}

bool Item::is_remote() {
  return remote;
}

void Item::push(item_info *data,size_t len) {
  if (connection && !remote) {
    data->id = this->id;
    //    printf("Sending event (id=%d, cls=%d, ev=%d)\n", data->id,data->cls,data->ev);
    connection->push(data,len);
  }
}

void Item::poll() {}

void Item::process(struct item_info *data,size_t len) {}

Item *new_Item(struct item_info *data,size_t len) {
  Ball *b;
  Pad *p;
  Stone *s;

  if (data->ev != EV_INIT) {
    printf("New_Item can only process INIT events.\n");
    return NULL;
  }

  switch (data->cls) {
  case CLS_STONE:
    s = new_stone(((struct stone_info*)data)->ch, data->id);
    s->process(data,len);
    level->add_stone(s);
    return s;
  case CLS_FIELD:
    level->process(data,len);
    return NULL;
  case CLS_PAD:
    p = new Pad(data->id);
    p->process(data,len);
    level->add_pad(p);
    return p;
  case CLS_BALL:
    b = new Ball(data->id);
    b->process(data,len);
    level->add_ball(b);
    return b;
  default:
    printf("Unknown class %d\n",data->cls);
    return NULL;
  }
}
