#include "littlecube.h"
#include "main.h"

/** Copyright (C) 2010 Maxime Gamboni. See COPYING for
    copying/warranty info. */

Littlecube::Littlecube(unsigned int id) : Item(id) {}
Littlecube::~Littlecube() {}

void Littlecube::init(double x,double y,double r,bool triggers, bool gravity) {

  // printf("little cube has radius %f\n",r);
  this->triggers = triggers;
  this->x = x;
  this->y = y;
  this->vx = 0;
  this->vy = 0;
  this->r = r;
  this->dr = 0;
  this->gravity=gravity;
  this->ccount = 0;
}

void Littlecube::move(double x,double y) {
  this->x = x;
  this->y = y;
}

void Littlecube::set_speed(double vx,double vy) {
  this->vx = vx;
  this->vy = vy;
}

/* The radius will increase by dr every second (typically dr is
   negative) */
void Littlecube::set_delta_radius(double dr) {
  this->dr = dr;
}

bool Littlecube::has_vanished() {
  return r == 0;
}

bool Littlecube::update(double t,Level *lvl) {

  double n[2];
  double v[2];

  bool coll=false;

  if (ccount>=30 || (!lvl->is_in_level(x,y))) {
    if (ccount>=30) {
      ccount=30;
      vx=0;vy=0;
    } else
      vy *= 1+t;
    set_delta_radius(-0.2);
  } else if (ccount) ccount--;

  if (gravity) vy -= t*2;

  // printf("pre-radius %f\n",r);

  r += dr*t;

  // printf("post-radius %f\n",r);

  if (r<0) r=0;

  if (vx==0 && vy==0) return false;

  lvl->reset_coll();

  for (int i=0;i<5;i++) {
    /* Move until collision point (if any) */
    double d = lvl->check_collision(x,y,vx,vy,t,n,v,NULL,triggers,this);
    if (d==2) { // no collision
      x += vx*t; y += vy*t; {
	ccount += i;
	return coll;
      }
    }

    /* obstacle speed relative to this, along the normal (lateral
       movement is ignored) */
    double os = (v[0]-vx)*n[0] + (v[1]-vy)*n[1];

    if (os < 0) {
      // printf("escaping...\n");
      /* The normal is pointing forward, i.e. we are actually getting
	 out of a stone or something. Then just move forward a tiny
	 bit too much to avoid getting stuck */
      x += vx*d*t*1.001; y += vy*d*t*1.001;
    } else {
    //    printf("_colln=(%f;%f),collv=(%f;%f),collov=(%f;%f)",n[0],n[1],v[0],v[1],dx,dy);
      x += vx*d*t*0.999f; y += vy*d*t*0.999f; // stop just before the collision

      coll = true;
      /* The obstacle is considered infinitely heavy (or we are
	 infinitely light) so add twice the obstacle's speed to
	 ours */
      vx += 2*os*n[0]; vy += 2*os*n[1];
    }
    /* remaining trajectory */
    t -= d*t;
  }
  ccount += 5;
  // printf("stuck somewhere...\n");
  return coll;
}

void Littlecube::look_at_me() {
  lookAt(x,y,dr!=0);
}

void Littlecube::render() {
  if (has_vanished()) return;

  //    glEnable(GL_LIGHTING);
  glPushMatrix(); {
    glTranslatef(x-r/2,y-r/2,0);

    render_cube(r,r,r);

  } glPopMatrix();

  //  glDisable(GL_LIGHTING);

}
