#include <math.h>

/** Copyright (C) 2010 Maxime Gamboni. See COPYING for
    copying/warranty info. */

#include "base.h"
#include "main.h"
#include "powerup.h"
#include "pad.h"
#include "level.h"

Powerup::Powerup() {}

Powerup::~Powerup() {}

bool Powerup::init(double x,double y,int *v,GLuint tex) {
  this->x = x;
  this->y = y;
  this->v[0] = v[0];
  this->v[1] = v[1];

  this->angle = 0;
  this->tex = tex;

  return true;
}

double Powerup::getX() {
  return x;
}

double Powerup::getY() {
  return y;
}

GLuint Powerup::getTex() {
  return tex;
}

bool Powerup::update(double t,Level *lvl) {
  angle += t*2;
  x += v[0]*t;
  y += v[1]*t;
  Pad *p;

  /* dividing speed by t to check the entire height of the
     powerup */
  if ((lvl->check_collision(x+v[0]*0.6,y+v[1]*0.6,
			    -v[0]*1.2/t,-v[1]*1.2/t,
			    t,0,0,&p,false) < 1.5) && p) {
    if (tex == wide_tex) {
      p->grow();
    } else if (tex == fire_tex) {
      p->fire();
    } else if (tex == extra_tex) {
      lvl->extra_ball(p);
    }
    return true;
  } else if (!lvl->is_in_level(x,y)) {
    return true;
  } else {
    return false;
  }
}

void Powerup::render() {
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, tex);

  glTranslatef(x,y,0);
  glColor3f(1,1,1);
  
  if (v[0]) glRotatef(90,0,0,1);
  glRotatef(angle RADIANS, 1,0,0);
  
  glBegin(GL_QUAD_STRIP);
  for (int i=0;i<7;i++) {
    double c = cos(i*PI/3);
    double s = sin(i*PI/3);

    glNormal3f(0,s,c);
  
    glTexCoord2f(0,1-i/3.0f);
    glVertex3f(-0.5f,s*0.4f,c*0.4f);

    glTexCoord2f(1,1-i/3.0f);
    glVertex3f(0.5f,s*0.4f,c*0.4f);
  }
  glEnd();
  glDisable(GL_TEXTURE_2D);

  glBegin(GL_POLYGON);
  glNormal3f(1,0,0);
  for (int i=0;i<6;i++) {
    double c = cos(i*PI/3);
    double s = sin(i*PI/3);
    glVertex3f(0.5f,s*0.4f,c*0.4f);
  }
  glEnd();

  glBegin(GL_POLYGON);
  glNormal3f(-1,0,0);
  for (int i=0;i<6;i++) {
    double c = cos(i*PI/3);
    double s = sin(i*PI/3);
    glVertex3f(-0.5f,s*0.4f,c*0.4f);
  }
  glEnd();
  
}

