#include "sound.h"

/** Copyright (C) 2010 Maxime Gamboni. See COPYING for
    copying/warranty info. */

/* These hold the currently playing sound (global variable) */
#define NUM_SOUNDS 4
struct sample {
    Uint8 *data;
    Uint32 dpos;
    Uint32 dlen;
} sounds[NUM_SOUNDS];

Sound::Sound() {
  dlen=0;
}

Sound::~Sound() {
  clean();
}

void Sound::clean() {
  if (dlen)
    free(cvt.buf);
  dlen=0;
}

bool Sound::loadWAV(const char *fn) {
  Uint8 *data;

  clean();
  if ( SDL_LoadWAV(fn, &wave, &data, &dlen) == NULL ) {
    fprintf(stderr, "Couldn't load %s: %s\n", fn, SDL_GetError());
    return false;
  }
  SDL_BuildAudioCVT(&cvt, wave.format, wave.channels, wave.freq,
		    AUDIO_S16,   2,             22050);
  cvt.buf = (Uint8*)malloc(dlen*cvt.len_mult);
  memcpy(cvt.buf, data, dlen);
  cvt.len = dlen;
  SDL_ConvertAudio(&cvt);
  SDL_FreeWAV(data);

  return true;
}

void Sound::play() {
  int index;
  if (!dlen) return; // no sound loaded

  /* Look for an empty (or finished) sound slot */
  for (index=0; index<NUM_SOUNDS; ++index ) {
    if ( sounds[index].dpos == sounds[index].dlen ) {
      break;
    }
  }
  if ( index == NUM_SOUNDS ) return; // no available slot

  /* Put the sound data in the slot (it starts playing immediately) */
  SDL_LockAudio();
  sounds[index].data = cvt.buf;
  sounds[index].dlen = cvt.len_cvt;
  sounds[index].dpos = 0;
  SDL_UnlockAudio();
}


void mixaudio(void *unused, Uint8 *stream, int len) {
  int i;
  int amount;
  
  for ( i=0; i<NUM_SOUNDS; ++i ) {
    amount = (sounds[i].dlen-sounds[i].dpos);
    if ( amount > len ) {
      amount = len;
    }
    SDL_MixAudio(stream, &sounds[i].data[sounds[i].dpos], amount, SDL_MIX_MAXVOLUME);
    sounds[i].dpos += amount;
  }
}
